﻿package
{
	import fl.controls.TextArea;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	import view.AutoBtn;
	import view.LightBtn;
	import view.MinusBtn;
	import view.PlusBtn;
	import view.StepBtn;
	
	/**
	 * PovEditメインクラス
	 * @author nomura masayuki
	 */
	public class PovEdit extends MovieClip
	{
		public static const STAGESIZE_W:uint = 19; // LEDステージ横サイズ
		public static const STAGESIZE_H:uint = 7;  // LEDステージ縦サイズ（JapaninoのPOVはLEDが7つ）
		public static const STAGEBTNSIZE_W:Number = 16; // LightBtnシンボルのWidth
		public static const STAGEBTNSIZE_H:Number = 16; // LightBtnシンボルのHeight
		
		private var lightArray:Array = []; // LightBtnインスタンスを格納する配列
		private var dataArray:Array = [];  // 各ステップでのLEDの点灯情報
		private var stepArray:Array = [];  // StepBtnインスタンスを格納する配列
		private var stepCount:uint = 0;    // 現在再生中のステップ
		private static const STEPCOUNT_MAX:uint = 14; // 最大ステップ数
		
		public var textArea:TextArea; // 文字列に出力
		public var plusBtn:PlusBtn;   // ステップ追加ボタン
		public var minusBtn:MinusBtn; // ステップ削除ボタン
		public var autoBtn:AutoBtn;   // アニメーション再生ボタン
		
		private var autoTimer:Timer; // アニメーションタイマー
		private static const TIMER_TIME:uint = 100; // アニメーション速度(ミリ秒)
		
		/**
		 * コンストラクタ
		 */
		public function PovEdit()
		{
			if (stage)
				f_init();
			else
				addEventListener(Event.ADDED_TO_STAGE, f_init);								 
		}
		
		/**
		 * 初期化処理
		 * @param	p_ev
		 */
		private function f_init(p_ev:Event = null):void
		{
			if (p_ev) removeEventListener(Event.ADDED_TO_STAGE, f_init);
			lightArray = [];
			initStage();
			addStep();
			lightClickHandler(null);
			
			// イベントリスナ登録
			plusBtn.addEventListener(PlusBtn.PLUSBTN_CLICK, plusBtnClickHandler);
			minusBtn.addEventListener(MinusBtn.MINUSBTN_CLICK, minusBtnClickHandler);
			autoBtn.addEventListener(AutoBtn.AUTOSTART, autoStartHandler);
			autoBtn.addEventListener(AutoBtn.AUTOSTOP, autoStopHandler);
			
			// アニメーション設定
			autoTimer = new Timer(TIMER_TIME); // アニメーションスピードを0.1秒に
			autoTimer.addEventListener(TimerEvent.TIMER, autoTimerHandler);
		}
		
		/**
		 * 新規ステップをステップ末尾に追加
		 */
		private function addStep():void
		{
			dataArray[stepCount] = [];
			for (var i:uint = 0; i < STAGESIZE_W; i++)
			{
				dataArray[stepCount][i] = [];
				for (var j:uint = 0; j < STAGESIZE_H; j++)
				{
					dataArray[stepCount][i][j] = 0;
					LightBtn(lightArray[i][j]).reset();
				}
			}
			
			// ステップボタンの追加
			var stepBtn:StepBtn = new StepBtn(stepCount);
			stepBtn.x = stepCount * 20 + 15;
			stepBtn.y = 294;
			stepBtn.addEventListener(StepBtn.STEP_CLICK, stepClickHandler);
			addChild(stepBtn);
			stepBtn.setCurrent();
			stepArray[stepCount] = stepBtn;
			
			// ステップ追加ボタンの表示調整
			plusBtn.x = stepBtn.x + 20;
			if (stepCount >= STEPCOUNT_MAX) plusBtn.visible = false;
		}
		
		/**
		 * LEDステージ初期化
		 */
		private function initStage():void
		{
			for (var i:uint = 0; i < STAGESIZE_W; i++)
			{
				lightArray[i] = [];
				for (var j:uint = 0; j < STAGESIZE_H; j++)
				{
					var lightBtn = new LightBtn();
					lightBtn.x = i * STAGEBTNSIZE_W + i + 15;
					lightBtn.y = j * STAGEBTNSIZE_H + j + 80;
					
					lightBtn.addEventListener(LightBtn.LIGHT_CHANGE, lightClickHandler);
					
					addChild(lightBtn);
					lightArray[i][j] = lightBtn;
				}
			}
		}
		
		/**
		 * LEDクリックハンドラ
		 * @param	p_ev
		 */
		private function lightClickHandler(p_ev:Event):void
		{
			var str:String = "";
			for (var i:uint = 0; i < STAGESIZE_W; i++)
			{
				var lineStr:String = "";
				for (var j:uint = 0; j < STAGESIZE_H; j++)
				{
					// on/off切り替え
					if (LightBtn(lightArray[i][STAGESIZE_H - j - 1]).on)
					{
						lineStr += "1";
						dataArray[stepCount][i][j] = 1;
					}
					else
					{
						lineStr += "0";
						dataArray[stepCount][i][j] = 0;
					}
				}
				str += "0b0" + lineStr + ",\n";
			}
			// 結果を文字列で表示
			textArea.text = str + "0b10000000";
		}
		
		/**
		 * ステップボタンを通常（非カレント）表示に戻す
		 */
		private function resetStepBtn():void
		{
			for (var i:uint = 0; i < stepArray.length; i++)
			{
				StepBtn(stepArray[i]).reset();
			}
			
		}
		
		/**
		 * ステップ追加ボタンハンドラ
		 * @param	p_ev
		 */
		private function plusBtnClickHandler(p_ev:Event):void
		{
			if (autoBtn.on) return; // アニメーション中は追加できない
			
			resetStepBtn();
			stepCount++;
			addStep();
		}
		
		/**
		 * ステップ削除ボタンハンドラ
		 * @param	p_ev
		 */
		private function minusBtnClickHandler(p_ev:Event):void
		{
			if (autoBtn.on) return; // アニメーション中は削除できない
			if (stepArray.length == 1) return; // ステップ数が1の場合は削除できない
			
			removeChild(stepArray.pop());
			dataArray.splice(stepCount, 1);
			
			if (stepCount >= stepArray.length)
				stepCount = stepArray.length - 1;
			changeStep(stepCount);
			
			// ステップ追加ボタンの表示調整
			if (stepCount < STEPCOUNT_MAX) plusBtn.visible = true;
			plusBtn.x = StepBtn(stepArray[stepArray.length - 1]).x + 20;
		}
		
		/**
		 * ステップボタンハンドラ
		 * @param	p_ev
		 */
		private function stepClickHandler(p_ev:Event):void
		{
			var _step:uint = StepBtn(p_ev.target).step;
			changeStep(_step);
		}
		
		/**
		 * 表示ステップの変更
		 * @param	_step 表示するステップ番号
		 */
		private function changeStep(_step:uint):void
		{
			resetStepBtn();
			stepCount = _step;
			StepBtn(stepArray[stepCount]).setCurrent();
			
			for (var i:uint = 0; i < STAGESIZE_W; i++)
			{
				for (var j:uint = 0; j < STAGESIZE_H; j++)
				{
					if (dataArray[stepCount][i][j] == 1)
					{
						LightBtn(lightArray[i][STAGESIZE_H - j - 1]).setOn();
					}
					else
					{
						LightBtn(lightArray[i][STAGESIZE_H - j - 1]).reset();
					}
				}
			}
			
			lightClickHandler(null);
		}
		
		/**
		 * アニメーションボタンハンドラ(再生)
		 * @param	p_ev
		 */
		private function autoStartHandler(p_ev:Event):void
		{
			autoTimer.start();
		}
		
		/**
		 * アニメーションボタンハンドラ(停止)
		 * @param	p_ev
		 */
		private function autoStopHandler(p_ev:Event):void
		{
			autoTimer.reset();
		}
		
		/**
		 * アニメーションタイマーハンドラ
		 * @param	p_ev
		 */
		private function autoTimerHandler(p_ev:TimerEvent):void
		{
			stepCount++;
			if (stepCount >= dataArray.length)
			{
				stepCount = 0;
			}
			
			changeStep(stepCount);
		}
	}
}